@@ -1,6 +1,6 @@
 /*
      This file is part of anastasis-gtk.
-     Copyright (C) 2020, 2021 Anastasis SARL
+     Copyright (C) 2020, 2021, 2022 Anastasis SARL
 
      Anastasis is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -35,11 +35,22 @@
  */
 struct GNUNET_GTK_MainLoop *AG_ml;
 
+/**
+ * Active policy discovery job, or NULL.
+ */
+struct ANASTASIS_PolicyDiscovery *AG_pd;
+
 /**
  * Our configuration.
  */
 const struct GNUNET_CONFIGURATION_Handle *AG_cfg;
 
+/**
+ * Application ID to include in the user attributes.
+ * (-a option).
+ */
+char *AG_application_id;
+
 /**
  * Closure for #GNUNET_CURL_gnunet_scheduler_reschedule().
  */
@@ -53,7 +64,7 @@ struct GNUNET_CONTAINER_MultiHashMap *AG_entry_attributes;
 /**
  * Curl context for communication with taler backend
  */
-static struct GNUNET_CURL_Context *ctx;
+struct GNUNET_CURL_Context *AG_ctx;
 
 /**
  * Handle to an ongoing action.
@@ -63,12 +74,7 @@ struct ANASTASIS_ReduxAction *AG_ra;
 /**
  * Handle to an ongoing background action.
  */
-struct ANASTASIS_ReduxAction *AG_long_action;
-
-/**
- * Handle to task to reschedule #AG_long_action.
- */
-struct GNUNET_SCHEDULER_Task *AG_long_task;
+struct ANASTASIS_LongAction AG_lacs[ANASTASIS_LP_CNT];
 
 
 /**
@@ -80,15 +86,22 @@ json_t *AG_redux_state;
 void
 AG_stop_long_action (void)
 {
-  if (NULL != AG_long_action)
+  for (enum ANASTASIS_LongActionKind i = 0;
+       i < ANASTASIS_LP_CNT;
+       i++)
   {
-    ANASTASIS_redux_action_cancel (AG_long_action);
-    AG_long_action = NULL;
-  }
-  if (NULL != AG_long_task)
-  {
-    GNUNET_SCHEDULER_cancel (AG_long_task);
-    AG_long_task = NULL;
+    struct ANASTASIS_LongAction *la = &AG_lacs[i];
+
+    if (NULL != la->ra)
+    {
+      ANASTASIS_redux_action_cancel (la->ra);
+      la->ra = NULL;
+    }
+    if (NULL != la->task)
+    {
+      GNUNET_SCHEDULER_cancel (la->task);
+      la->task = NULL;
+    }
   }
 }
 
@@ -104,6 +117,11 @@ shutdown_task (void *cls)
   (void) cls;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Shutdown initiated\n");
+  if (NULL != AG_pd)
+  {
+    ANASTASIS_policy_discovery_stop (AG_pd);
+    AG_pd = NULL;
+  }
   ANASTASIS_redux_done ();
   if (NULL != AG_ra)
   {
@@ -111,10 +129,10 @@ shutdown_task (void *cls)
     AG_ra = NULL;
   }
   AG_stop_long_action ();
-  if (NULL != ctx)
+  if (NULL != AG_ctx)
   {
-    GNUNET_CURL_fini (ctx);
-    ctx = NULL;
+    GNUNET_CURL_fini (AG_ctx);
+    AG_ctx = NULL;
   }
   if (NULL != rc)
   {
@@ -220,10 +238,10 @@ run (void *cls)
   GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
                                  NULL);
   /* initialize HTTP client */
-  ctx = GNUNET_CURL_init (&GNUNET_CURL_gnunet_scheduler_reschedule,
-                          &rc);
-  rc = GNUNET_CURL_gnunet_rc_create (ctx);
-  ANASTASIS_redux_init (ctx);
+  AG_ctx = GNUNET_CURL_init (&GNUNET_CURL_gnunet_scheduler_reschedule,
+                             &rc);
+  rc = GNUNET_CURL_gnunet_rc_create (AG_ctx);
+  ANASTASIS_redux_init (AG_ctx);
   if (0 != argc)
     AG_load (argv[0]);
 }
@@ -241,10 +259,16 @@ main (int argc,
       char *const *argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_string ('A',
+                                 "application",
+                                 "ID",
+                                 "set the application ID",
+                                 &AG_application_id),
     GNUNET_GETOPT_OPTION_END
   };
   int ret;
 
+  AG_application_id = GNUNET_strdup ("anastasis-standalone");
   if (GNUNET_OK !=
       GNUNET_GTK_main_loop_start ("anastasis-gtk",
                                   "GTK GUI for Anastasis",
@@ -256,6 +280,7 @@ main (int argc,
     ret = 1;
   else
     ret = 0;
+  GNUNET_free (AG_application_id);
   return ret;
 }
 
