@@ -318,6 +318,7 @@ url_add_button_clicked_cb (GtkButton *button,
                                      AG_PMC_PROVIDER_ENABLED, true,
                                      AG_PMC_PROVIDER_SENSITIVE, false,
                                      AG_PMC_PROVIDER_NOT_SENSITIVE, true,
+                                     AG_PMC_PROVIDER_NAME, url,
                                      -1);
   gtk_entry_set_text (entry,
                       "");
@@ -325,7 +326,11 @@ url_add_button_clicked_cb (GtkButton *button,
 
 
 /**
- * FIXME.
+ * The user changed the URL of a provider to be possibly added.
+ * Validate the syntax and if it is a valid URL, enable the button.
+ *
+ * @param entry the edited widget
+ * @param user_data the dialog builder
  */
 void
 url_entry_changed_cb (GtkEntry *entry,
@@ -395,8 +400,10 @@ edit_provider_dialog_response_cb (GtkDialog *dialog,
                        args,
                        url,
                        GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_bool ("disabled",
-                                                ! enabled))));
+                         GNUNET_JSON_pack_string ("status",
+                                                  enabled
+                                                  ? "not-contacted"
+                                                  : "disabled"))));
       g_free (url);
     }
     while (gtk_tree_model_iter_next (tm,
@@ -444,26 +451,33 @@ anastasis_gtk_edit_provider_list_clicked_cb (GtkButton *object,
     const json_t *provider;
     json_object_foreach (providers, url, provider)
     {
-      uint32_t http_code;
-      uint32_t ec;
+      uint32_t http_code = 0;
+      uint32_t ec = TALER_EC_NONE;
       struct TALER_Amount ll;
-      bool disabled = false;
+      const char *status;
+      const char *name = NULL;
       struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_string ("status",
+                                 &status),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_uint32 ("http_status",
-                                   &http_code)),
+                                   &http_code),
+          NULL),
         GNUNET_JSON_spec_mark_optional (
-          GNUNET_JSON_spec_bool ("disabled",
-                                 &disabled)),
+          GNUNET_JSON_spec_string ("business_name",
+                                   &name),
+          NULL),
         GNUNET_JSON_spec_mark_optional (
           TALER_JSON_spec_amount_any ("liability_limit",
-                                      &ll)),
+                                      &ll),
+          NULL),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_uint32 ("error_code",
-                                   &ec)),
+                                   &ec),
+          NULL),
         GNUNET_JSON_spec_end ()
       };
-      char *status;
+      char *status_str;
       const char *color;
       bool sensitive = false;
       const char *ll_s = NULL;
@@ -483,9 +497,10 @@ anastasis_gtk_edit_provider_list_clicked_cb (GtkButton *object,
         continue;
       }
       if ( (MHD_HTTP_OK == http_code) &&
-           (! disabled) )
+           (0 != strcmp (status,
+                         "disabled")) )
       {
-        status = GNUNET_strdup (_ ("available"));
+        status_str = GNUNET_strdup (_ ("available"));
         color = "green";
         sensitive = true;
         if (GNUNET_OK ==
@@ -495,24 +510,26 @@ anastasis_gtk_edit_provider_list_clicked_cb (GtkButton *object,
           GNUNET_break (0);
       }
       else if ( (0 == http_code) &&
-                (! disabled) )
+                (0 != strcmp (status,
+                              "disabled")) )
       {
-        GNUNET_asprintf (&status,
+        GNUNET_asprintf (&status_str,
                          _ ("Network failure: %s (#%u)"),
                          TALER_ErrorCode_get_hint (ec),
                          (unsigned int) ec);
         color = "red";
       }
-      else if (disabled)
+      else if (0 == strcmp (status,
+                            "disabled"))
       {
-        GNUNET_asprintf (&status,
+        GNUNET_asprintf (&status_str,
                          _ ("disabled"));
         color = "blue";
         sensitive = true;
       }
       else
       {
-        GNUNET_asprintf (&status,
+        GNUNET_asprintf (&status_str,
                          _ ("HTTP %s (%u): %s (#%u)"),
                          MHD_get_reason_phrase_for (http_code),
                          (unsigned int) http_code,
@@ -520,19 +537,23 @@ anastasis_gtk_edit_provider_list_clicked_cb (GtkButton *object,
                          (unsigned int) ec);
         color = "red";
       }
+      if (NULL == name)
+        name = url;
       gtk_list_store_insert_with_values (
         ls,
         NULL,
         -1,
         AG_PMC_PROVIDER_URL, url,
-        AG_PMC_PROVIDER_STATUS, status,
+        AG_PMC_PROVIDER_STATUS, status_str,
         AG_PMC_PROVIDER_STATUS_COLOR, color,
         AG_PMC_PROVIDER_LIABILITY_LIMIT, ll_s,
-        AG_PMC_PROVIDER_ENABLED, ! disabled,
+        AG_PMC_PROVIDER_ENABLED, (0 != strcmp (status,
+                                               "disabled")),
         AG_PMC_PROVIDER_SENSITIVE, sensitive,
         AG_PMC_PROVIDER_NOT_SENSITIVE, ! sensitive,
+        AG_PMC_PROVIDER_NAME, name,
         -1);
-      GNUNET_free (status);
+      GNUNET_free (status_str);
     }
   }
   ad = GTK_WIDGET (gtk_builder_get_object (builder,
