@@ -22,6 +22,7 @@ function cleanup()
 {
     for n in `jobs -p`
     do
+        kill -SIGCONT $n # in case one provider was suspended
         kill $n 2> /dev/null || true
     done
     rm -rf $CONF $CONF_4 $WALLET_DB $R1FILE $R2FILE $B1FILE $B2FILE $TMP_DIR
@@ -49,8 +50,7 @@ function prepare_sandbox_account() {
         --account-name=$4 \
         --ebics-user-id=$1 \
         --ebics-host-id=$EBICS_HOST \
-        --ebics-partner-id=$2 \
-        --currency=EUR
+        --ebics-partner-id=$2
   echo " OK"
 }
 
@@ -164,7 +164,7 @@ then
 fi
 
 echo -n "Testing for libeufin-cli"
-if libeufin-cli --version > /dev/null
+if test $1 = 'fees' && libeufin-cli --version > /dev/null 2>/dev/null
 then
     echo " FOUND"
     IBAN_CREDIT=`anastasis-config -c $CONF_4 -s authorization-iban -o CREDIT_IBAN`
@@ -180,7 +180,7 @@ then
         exit_skip "Could not launch Nexus"
     fi
     echo -n "."
-    libeufin-sandbox serve &> sandbox.log &
+    libeufin-sandbox serve --no-auth &> sandbox.log &
     sandbox_pid=$!
     if ! curl -s --retry 5 --retry-connrefused $SANDBOX_URL > /dev/null; then
         exit_skip "Could not launch Sandbox"
@@ -333,7 +333,7 @@ then
     rm -rf $DATA_DIR
 
     # obtain key configuration data
-    MASTER_PRIV_FILE=`taler-config -f -c $CONF -s EXCHANGE -o MASTER_PRIV_FILE`
+    MASTER_PRIV_FILE=`taler-config -f -c $CONF -s EXCHANGE-OFFLINE -o MASTER_PRIV_FILE`
     MASTER_PRIV_DIR=`dirname $MASTER_PRIV_FILE`
     mkdir -p $MASTER_PRIV_DIR
     gnunet-ecc -g1 $MASTER_PRIV_FILE > /dev/null 2> /dev/null
@@ -380,6 +380,7 @@ then
     taler-bank-manage-testing $CONF postgres:///$TALER_DB serve > taler-bank.log 2> taler-bank.err &
     taler-exchange-secmod-eddsa -c $CONF 2> taler-exchange-secmod-eddsa.log &
     taler-exchange-secmod-rsa -c $CONF 2> taler-exchange-secmod-rsa.log &
+    taler-exchange-secmod-cs -c $CONF 2> taler-exchange-secmod-cs.log &
     taler-exchange-httpd -c $CONF 2> taler-exchange-httpd.log &
     taler-merchant-httpd -c $CONF -L INFO 2> taler-merchant-httpd.log &
     taler-exchange-wirewatch -c $CONF 2> taler-exchange-wirewatch.log &
@@ -394,10 +395,17 @@ echo -n "Launching anastasis services ..."
 # PREFIX="valgrind --log-file=anastasis-httpd.%p.log"
 PREFIX=""
 $PREFIX anastasis-httpd -L INFO -c $CONF_1 2> anastasis-httpd_1.log &
+PPID_1=$!
 $PREFIX anastasis-httpd -L INFO -c $CONF_2 2> anastasis-httpd_2.log &
+PPID_2=$!
 $PREFIX anastasis-httpd -L INFO -c $CONF_3 2> anastasis-httpd_3.log &
+PPID_3=$!
 $PREFIX anastasis-httpd -L INFO -c $CONF_4 2> anastasis-httpd_4.log &
-
+PPID_4=$!
+export PPID_1
+export PPID_2
+export PPID_3
+export PPID_4
 
 if test $1 = 'fees'
 then
@@ -444,12 +452,13 @@ then
 
     echo -n "Setting up keys ..."
     taler-exchange-offline -c $CONF \
-                           download \
-                           sign \
-                           enable-account payto://x-taler-bank/localhost/Exchange \
-                           enable-auditor $AUDITOR_PUB $AUDITOR_URL "TESTKUDOS Auditor" \
-                           wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 \
-                           upload &> taler-exchange-offline.log
+      download \
+      sign \
+      enable-account payto://x-taler-bank/localhost/Exchange \
+      enable-auditor $AUDITOR_PUB $AUDITOR_URL "TESTKUDOS Auditor" \
+      wire-fee now x-taler-bank TESTKUDOS:0.01 TESTKUDOS:0.01 TESTKUDOS:0.01 \
+      global-fee now TESTKUDOS:0.01 TESTKUDOS:0.01 TESTKUDOS:0.01 TESTKUDOS:0.01 1h 1h 1year 5 \
+      upload &> taler-exchange-offline.log
 
     echo -n "."
 
@@ -509,7 +518,7 @@ then
     echo -n "Configuring merchant instance ..."
     # Setup merchant
 
-    curl -H "Content-Type: application/json" -X POST -d '{"auth":{"method":"external"},"payto_uris":["payto://x-taler-bank/localhost/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_ms" : 3600000},"default_pay_delay":{"d_ms": 3600000}}' http://localhost:9966/private/instances
+    curl -H "Content-Type: application/json" -X POST -d '{"auth":{"method":"external"},"payto_uris":["payto://x-taler-bank/localhost/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_ms" : 3600000},"default_pay_delay":{"d_ms": 3600000}}' http://localhost:9966/management/instances
 
     echo " OK"
 
@@ -547,6 +556,8 @@ then
     export WALLET_DB
 fi
 
+echo '- use kill -SIGSTOP $PPID_$NUM ($NUM: 1-4) to suspend providers'
+
 bash
 
 exit 0
